#if 0
DEF PLATFORM_UNIX = 0
DEF PLATFORM_WIN = 0
#endif
#define PLATFORM_UNIX 1


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  # Constants shared by multiple CSS Box Alignment properties
  #
  # These constants match Gecko's `NS_STYLE_ALIGN_*` constants.
  ctypedef struct StyleAlignFlags:
    uint8_t bits;
  # 'auto'
  const StyleAlignFlags StyleAlignFlags_AUTO # = <StyleAlignFlags>{ <uint8_t>0 }
  # 'normal'
  const StyleAlignFlags StyleAlignFlags_NORMAL # = <StyleAlignFlags>{ <uint8_t>1 }
  # 'start'
  const StyleAlignFlags StyleAlignFlags_START # = <StyleAlignFlags>{ <uint8_t>(1 << 1) }
  # 'end'
  const StyleAlignFlags StyleAlignFlags_END # = <StyleAlignFlags>{ <uint8_t>(1 << 2) }
  const StyleAlignFlags StyleAlignFlags_ALIAS # = <StyleAlignFlags>{ <uint8_t>(StyleAlignFlags_END).bits }
  # 'flex-start'
  const StyleAlignFlags StyleAlignFlags_FLEX_START # = <StyleAlignFlags>{ <uint8_t>(1 << 3) }
  const StyleAlignFlags StyleAlignFlags_MIXED # = <StyleAlignFlags>{ <uint8_t>(((1 << 4) | (StyleAlignFlags_FLEX_START).bits) | (StyleAlignFlags_END).bits) }
  const StyleAlignFlags StyleAlignFlags_MIXED_SELF # = <StyleAlignFlags>{ <uint8_t>(((1 << 5) | (StyleAlignFlags_FLEX_START).bits) | (StyleAlignFlags_END).bits) }
  IF PLATFORM_WIN:
    const StyleAlignFlags StyleAlignFlags_PLATFORM_BIT # = <StyleAlignFlags>{ <uint8_t>(1 << 6) }
  IF PLATFORM_UNIX:
    const StyleAlignFlags StyleAlignFlags_PLATFORM_BIT # = <StyleAlignFlags>{ <uint8_t>(1 << 7) }

  # An arbitrary identifier for a native (OS compositor) surface
  ctypedef struct StyleNativeSurfaceId:
    uint64_t _0;
  # A special id for the native surface that is used for debug / profiler overlays.
  const StyleNativeSurfaceId StyleNativeSurfaceId_DEBUG_OVERLAY # = <StyleNativeSurfaceId>{ UINT64_MAX }

  ctypedef struct StyleNativeTileId:
    StyleNativeSurfaceId surface_id;
    int32_t x;
    int32_t y;
  # A special id for the native surface that is used for debug / profiler overlays.
  const StyleNativeTileId StyleNativeTileId_DEBUG_OVERLAY # = <StyleNativeTileId>{ StyleNativeSurfaceId_DEBUG_OVERLAY, 0, 0 }

  void root(StyleAlignFlags flags, StyleNativeTileId tile);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef struct Opaque:
    pass

  cdef struct Option_____Opaque:
    pass

  cdef struct Foo:
    const Opaque *x;
    Opaque *y;
    void (*z)();
    void (**zz)();

  cdef union Bar:
    const Opaque *x;
    Opaque *y;
    void (*z)();
    void (**zz)();

  void root(const Opaque *a,
            Opaque *b,
            Foo c,
            Bar d,
            Option_____Opaque *e,
            void (*f)(const Opaque*));



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef int32_t (*VaListFnPtr)(int32_t count, ...);

  ctypedef int32_t (*VaListFnPtr2)(int32_t count, ...);

  cdef struct Interface_______i32_______i32_______va_list:
    int32_t (*fn1)(int32_t count, ...);

  int32_t va_list_test(int32_t count, ...);

  int32_t va_list_test2(int32_t count, ...);

  void va_list_fn_ptrs(int32_t (*fn1)(int32_t count, ...),
                       int32_t (*fn2)(int32_t count, ...),
                       VaListFnPtr fn3,
                       VaListFnPtr2 fn4,
                       Interface_______i32_______i32_______va_list fn5,
                       Interface_______i32_______i32_______va_list fn6);



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# nghttp2 - HTTP/2 C Library
#
# Copyright (c) 2020 nghttp2 contributors
# Copyright (c) 2020 ngtcp2 contributors
# Copyright (c) 2012 Tatsuhiro Tsujikawa
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:

# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# Generates API reference from C source code.

import re, sys, argparse, os.path

class FunctionDoc:
    def __init__(self, name, content, domain, filename):
        self.name = name
        self.content = content
        self.domain = domain
        if self.domain == 'function':
            self.funcname = re.search(r'(nghttp2_[^ )]+)\(', self.name).group(1)
        self.filename = filename

    def write(self, out):
        out.write('.. {}:: {}\n'.format(self.domain, self.name))
        out.write('\n')
        for line in self.content:
            out.write('    {}\n'.format(line))

class StructDoc:
    def __init__(self, name, content, members, member_domain):
        self.name = name
        self.content = content
        self.members = members
        self.member_domain = member_domain

    def write(self, out):
        if self.name:
            out.write('.. type:: {}\n'.format(self.name))
            out.write('\n')
            for line in self.content:
                out.write('    {}\n'.format(line))
            out.write('\n')
            for name, content in self.members:
                out.write('    .. {}:: {}\n'.format(self.member_domain, name))
                out.write('\n')
                for line in content:
                    out.write('        {}\n'.format(line))
            out.write('\n')

class EnumDoc:
    def __init__(self, name, content, members):
        self.name = name
        self.content = content
        self.members = members

    def write(self, out):
        if self.name:
            out.write('.. type:: {}\n'.format(self.name))
            out.write('\n')
            for line in self.content:
                out.write('    {}\n'.format(line))
            out.write('\n')
            for name, content in self.members:
                out.write('    .. enum:: {}\n'.format(name))
                out.write('\n')
                for line in content:
                    out.write('        {}\n'.format(line))
            out.write('\n')

class MacroDoc:
    def __init__(self, name, content):
        self.name = name
        self.content = content

    def write(self, out):
        out.write('''.. macro:: {}\n'''.format(self.name))
        out.write('\n')
        for line in self.content:
            out.write('    {}\n'.format(line))

class MacroSectionDoc:
    def __init__(self, content):
        self.content = content

    def write(self, out):
        out.write('\n')
        c = ' '.join(self.content).strip()
        out.write(c)
        out.write('\n')
        out.write('-' * len(c))
        out.write('\n\n')

class TypedefDoc:
    def __init__(self, name, content):
        self.name = name
        self.content = content

    def write(self, out):
        out.write('''.. type:: {}\n'''.format(self.name))
        out.write('\n')
        for line in self.content:
            out.write('    {}\n'.format(line))

def make_api_ref(infile):
    macros = []
    enums = []
    types = []
    functions = []
    while True:
        line = infile.readline()
        if not line:
            break
        elif line == '/**\n':
            line = infile.readline()
            doctype = line.split()[1]
            if doctype == '@function':
                functions.append(process_function('function', infile))
            elif doctype == '@functypedef':
                types.append(process_function('type', infile))
            elif doctype == '@struct' or doctype == '@union':
                types.append(process_struct(infile))
            elif doctype == '@enum':
                enums.append(process_enum(infile))
            elif doctype == '@macro':
                macros.append(process_macro(infile))
            elif doctype == '@macrosection':
                macros.append(process_macrosection(infile))
            elif doctype == '@typedef':
                types.append(process_typedef(infile))
    return macros, enums, types, functions

def output(
        title, indexfile, macrosfile, enumsfile, typesfile, funcsdir,
        macros, enums, types, functions):
    indexfile.write('''
{title}
{titledecoration}

.. toctree::
   :maxdepth: 1

   {macros}
   {enums}
   {types}
'''.format(
    title=title, titledecoration='='*len(title),
    macros=os.path.splitext(os.path.basename(macrosfile.name))[0],
    enums=os.path.splitext(os.path.basename(enumsfile.name))[0],
    types=os.path.splitext(os.path.basename(typesfile.name))[0],
))

    for doc in functions:
        indexfile.write('   {}\n'.format(doc.funcname))

    macrosfile.write('''
Macros
======
''')
    for doc in macros:
        doc.write(macrosfile)

    enumsfile.write('''
Enums
=====
''')
    for doc in enums:
        doc.write(enumsfile)

    typesfile.write('''
Types (structs, unions and typedefs)
====================================
''')
    for doc in types:
        doc.write(typesfile)

    for doc in functions:
        with open(os.path.join(funcsdir, doc.funcname + '.rst'), 'w') as f:
            f.write('''
{funcname}
{secul}

Synopsis
--------

*#include <nghttp2/{filename}>*

'''.format(funcname=doc.funcname, secul='='*len(doc.funcname),
           filename=doc.filename))
            doc.write(f)

def process_macro(infile):
    content = read_content(infile)
    line = infile.readline()
    macro_name = line.split()[1]
    return MacroDoc(macro_name, content)

def process_macrosection(infile):
    content = read_content(infile)
    return MacroSectionDoc(content)

def process_typedef(infile):
    content = read_content(infile)
    typedef = infile.readline()
    typedef = re.sub(r';\n$', '', typedef)
    typedef = re.sub(r'typedef ', '', typedef)
    return TypedefDoc(typedef, content)

def process_enum(infile):
    members = []
    enum_name = None
    content = read_content(infile)
    while True:
        line = infile.readline()
        if not line:
            break
        elif re.match(r'\s*/\*\*\n', line):
            member_content = read_content(infile)
            line = infile.readline()
            items = line.split()
            member_name = items[0].rstrip(',')
            if len(items) >= 3:
                member_content.insert(0, '(``{}``) '\
                                      .format(' '.join(items[2:]).rstrip(',')))
            members.append((member_name, member_content))
        elif line.startswith('}'):
            enum_name = line.rstrip().split()[1]
            enum_name = re.sub(r';$', '', enum_name)
            break
    return EnumDoc(enum_name, content, members)

def process_struct(infile):
    members = []
    struct_name = None
    content = read_content(infile)
    while True:
        line = infile.readline()
        if not line:
            break
        elif re.match(r'\s*/\*\*\n', line):
            member_content = read_content(infile)
            line = infile.readline()
            member_name = line.rstrip().rstrip(';')
            members.append((member_name, member_content))
        elif line.startswith('}') or\
                (line.startswith('typedef ') and line.endswith(';\n')):
            if line.startswith('}'):
                index = 1
            else:
                index = 3
            struct_name = line.rstrip().split()[index]
            struct_name = re.sub(r';$', '', struct_name)
            break
    return StructDoc(struct_name, content, members, 'member')

def process_function(domain, infile):
    content = read_content(infile)
    func_proto = []
    while True:
        line = infile.readline()
        if not line:
            break
        elif line == '\n':
            break
        else:
            func_proto.append(line)
    func_proto = ''.join(func_proto)
    func_proto = re.sub(r';\n$', '', func_proto)
    func_proto = re.sub(r'\s+', ' ', func_proto)
    func_proto = re.sub(r'NGHTTP2_EXTERN ', '', func_proto)
    func_proto = re.sub(r'typedef ', '', func_proto)
    filename = os.path.basename(infile.name)
    return FunctionDoc(func_proto, content, domain, filename)

def read_content(infile):
    content = []
    while True:
        line = infile.readline()
        if not line:
            break
        if re.match(r'\s*\*/\n', line):
            break
        else:
            content.append(transform_content(line.rstrip()))
    return content

def arg_repl(matchobj):
    return '*{}*'.format(matchobj.group(1).replace('*', '\\*'))

def transform_content(content):
    content = re.sub(r'^\s+\* ?', '', content)
    content = re.sub(r'\|([^\s|]+)\|', arg_repl, content)
    return content

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="Generate API reference")
    parser.add_argument('--title', default='API Reference',
                        help='title of index page')
    parser.add_argument('index', type=argparse.FileType('w'),
                        help='index output file')
    parser.add_argument('macros', type=argparse.FileType('w'),
                        help='macros section output file.  The filename should be macros.rst')
    parser.add_argument('enums', type=argparse.FileType('w'),
                        help='enums section output file.  The filename should be enums.rst')
    parser.add_argument('types', type=argparse.FileType('w'),
                        help='types section output file.  The filename should be types.rst')
    parser.add_argument('funcsdir',
                        help='functions doc output dir')
    parser.add_argument('files', nargs='+', type=argparse.FileType('r'),
                        help='source file')
    args = parser.parse_args()
    macros = []
    enums = []
    types = []
    funcs = []
    for infile in args.files:
        m, e, t, f = make_api_ref(infile)
        macros.extend(m)
        enums.extend(e)
        types.extend(t)
        funcs.extend(f)
    funcs.sort(key=lambda x: x.funcname)
    output(
        args.title,
        args.index, args.macros, args.enums, args.types, args.funcsdir,
        macros, enums, types, funcs)



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef enum:
    Nonzero,
    Evenodd,
  ctypedef uint8_t FillRule;

  cdef struct Style:
    FillRule rule;
  const FillRule Style_DEFAULT_RULE # = Nonzero
  const FillRule Style_ALL_RULES[2] # = [ Nonzero, Evenodd, ]

  const FillRule DEFAULT_FILL_RULE # = Nonzero

  const FillRule ALL_FILL_RULES[2] # = [ Nonzero, Evenodd, ]

  void root(FillRule rule, Style style);



#!/usr/bin/env python3
import sys

def name(i):
    if i < 0x20:
        return \
            ['NUL ', 'SOH ', 'STX ', 'ETX ', 'EOT ', 'ENQ ', 'ACK ', 'BEL ',
             'BS  ', 'HT  ', 'LF  ', 'VT  ', 'FF  ', 'CR  ', 'SO  ', 'SI  ',
             'DLE ', 'DC1 ', 'DC2 ', 'DC3 ', 'DC4 ', 'NAK ', 'SYN ', 'ETB ',
             'CAN ', 'EM  ', 'SUB ', 'ESC ', 'FS  ', 'GS  ', 'RS  ', 'US  '][i]
    elif i == 0x7f:
        return 'DEL '

for i in range(256):
    if chr(i) == ' ':
        sys.stdout.write('1 /* SPC  */, ')
    elif chr(i) == '\t':
        sys.stdout.write('1 /* HT   */, ')
    elif (0x21 <= i and i < 0x7f):
        sys.stdout.write('1 /* {}    */, '.format(chr(i)))
    elif 0x80 <= i:
        sys.stdout.write('1 /* {} */, '.format(hex(i)))
    else:
        sys.stdout.write('0 /* {} */, '.format(name(i)))
    if (i + 1)%4 == 0:
        sys.stdout.write('\n')



#!/usr/bin/env python

from collections import namedtuple

import argparse
import base64
import copy
import json
import subprocess
import sys
import urllib.parse
import urllib.request
import urllib.error

class Error(Exception):
    pass

class Version(object):
    def __init__(self, version):
        versions = version.split(sep='.')
        if len(versions) < 2 or len(versions) > 3:
            raise Error("Invalid version string '{}'".format(version))
        self.major = int(versions[0])
        self.minor = int(versions[1])
        self.revision = int(versions[2]) if len(versions) == 3 else 0

    def __str__(self):
        return '{}.{}.{}'.format(self.major, self.minor, self.revision)

    def __eq__(self, other):
        return self.major == other.major and self.minor == other.minor and self.revision == other.revision

def verify_version(version):
    expected = {
        'VERSION':      [ '"{}"'.format(version), None ],
        'VER_MAJOR':    [ str(version.major), None ],
        'VER_MINOR':    [ str(version.minor), None ],
        'VER_REVISION': [ str(version.revision), None ],
        'VER_PATCH':    [ '0', None ],
        'SOVERSION':    [ '"{}.{}"'.format(version.major, version.minor), None ],
    }

    # Parse CMakeLists
    with open('CMakeLists.txt') as f:
        for line in f.readlines():
            if line.startswith('project(libgit2 VERSION "{}"'.format(version)):
                break
        else:
            raise Error("cmake: invalid project definition")

    # Parse version.h
    with open('include/git2/version.h') as f:
        lines = f.readlines()

    for key in expected.keys():
        define = '#define LIBGIT2_{} '.format(key)
        for line in lines:
            if line.startswith(define):
                expected[key][1] = line[len(define):].strip()
                break
        else:
            raise Error("version.h: missing define for '{}'".format(key))

    for k, v in expected.items():
        if v[0] != v[1]:
            raise Error("version.h: define '{}' does not match (got '{}', expected '{}')".format(k, v[0], v[1]))

    with open('package.json') as f:
        pkg = json.load(f)

    try:
        pkg_version = Version(pkg["version"])
    except KeyError as err:
        raise Error("package.json: missing the field {}".format(err))

    if pkg_version != version:
        raise Error("package.json: version does not match (got '{}', expected '{}')".format(pkg_version, version))

def generate_relnotes(tree, version):
    with open('docs/changelog.md') as f:
        lines = f.readlines()

    if not lines[0].startswith('v'):
        raise Error("changelog.md: missing section for v{}".format(version))
    try:
        v = Version(lines[0][1:].strip())
    except:
        raise Error("changelog.md: invalid version string {}".format(lines[0].strip()))
    if v != version:
        raise Error("changelog.md: changelog version doesn't match (got {}, expected {})".format(v, version))
    if not lines[1].startswith('----'):
        raise Error("changelog.md: missing version header")
    if lines[2] != '\n':
        raise Error("changelog.md: missing newline after version header")

    for i, line in enumerate(lines[3:]):
        if not line.startswith('v'):
            continue
        try:
            Version(line[1:].strip())
            break
        except:
            continue
    else:
        raise Error("changelog.md: cannot find section header of preceding release")

    return ''.join(lines[3:i + 3]).strip()

def git(*args):
    process = subprocess.run([ 'git', *args ], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if process.returncode != 0:
        raise Error('Failed executing git {}: {}'.format(' '.join(args),  process.stderr.decode()))
    return process.stdout

def post(url, data, contenttype, user, password):
    request = urllib.request.Request(url, data=data)
    request.add_header('Accept', 'application/json')
    request.add_header('Content-Type', contenttype)
    request.add_header('Content-Length', len(data))
    request.add_header('Authorization', 'Basic ' + base64.b64encode('{}:{}'.format(user, password).encode()).decode())

    try:
        response = urllib.request.urlopen(request)
        if response.getcode() != 201:
            raise Error("POST to '{}' failed: {}".format(url, response.reason))
    except urllib.error.URLError as e:
        raise Error("POST to '{}' failed: {}".format(url, e))
    data = json.load(response)

    return data

def generate_asset(version, tree, archive_format):
    Asset = namedtuple('Asset', ['name', 'label', 'mimetype', 'data'])
    mimetype = 'application/{}'.format('gzip' if archive_format == 'tar.gz' else 'zip')
    return Asset(
        "libgit2-{}.{}".format(version, archive_format), "Release sources: libgit2-{}.{}".format(version, archive_format), mimetype,
        git('archive', '--format', archive_format, '--prefix', 'libgit2-{}/'.format(version), tree)
    )

def release(args):
    params = {
        "tag_name": 'v' + str(args.version),
        "name": 'libgit2 v' + str(args.version),
        "target_commitish": git('rev-parse', args.tree).decode().strip(),
        "body": generate_relnotes(args.tree, args.version),
    }
    assets = [
        generate_asset(args.version, args.tree, 'tar.gz'),
        generate_asset(args.version, args.tree, 'zip'),
    ]

    if args.dryrun:
        for k, v in params.items():
            print('{}: {}'.format(k, v))
        for asset in assets:
            print('asset: name={}, label={}, mimetype={}, bytes={}'.format(asset.name, asset.label, asset.mimetype, len(asset.data)))
        return

    try:
        url = 'https://api.github.com/repos/{}/releases'.format(args.repository)
        response = post(url, json.dumps(params).encode(), 'application/json', args.user, args.password)
    except Error as e:
        raise Error('Could not create release: ' + str(e))

    for asset in assets:
        try:
            url = list(urllib.parse.urlparse(response['upload_url'].split('{?')[0]))
            url[4] = urllib.parse.urlencode({ 'name': asset.name, 'label': asset.label })
            post(urllib.parse.urlunparse(url), asset.data, asset.mimetype, args.user, args.password)
        except Error as e:
            raise Error('Could not upload asset: ' + str(e))

def main():
    parser = argparse.ArgumentParser(description='Create a libgit2 release')
    parser.add_argument('--tree', default='HEAD', help='tree to create release for (default: HEAD)')
    parser.add_argument('--dryrun', action='store_true', help='generate release, but do not post it')
    parser.add_argument('--repository', default='libgit2/libgit2', help='GitHub repository to create repository in')
    parser.add_argument('--user', help='user to authenticate as')
    parser.add_argument('--password', help='password to authenticate with')
    parser.add_argument('version', type=Version, help='version of the new release')
    args = parser.parse_args()

    verify_version(args.version)
    release(args)

if __name__ == '__main__':
    try:
        main()
    except Error as e:
        print(e)
        sys.exit(1)



#define MUST_USE_FUNC __attribute__((warn_unused_result))
#define MUST_USE_STRUCT __attribute__((warn_unused))
#define MUST_USE_ENUM /* nothing */


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef enum:
    Owned_i32,
    None_i32,
  ctypedef uint8_t MaybeOwnedPtr_i32_Tag;

  cdef struct MaybeOwnedPtr_i32:
    MaybeOwnedPtr_i32_Tag tag;
    int32_t *owned;

  cdef struct OwnedPtr_i32:
    int32_t *ptr;

  MaybeOwnedPtr_i32 maybe_consume(OwnedPtr_i32 input);
